﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Controls.Menus;

namespace WpfApplication3
{
    /// <summary>
    /// Interaction logic for CheckTreeView.xaml
    /// </summary>
    public partial class CheckTreeView : UserControl
    {
        private bool _inCheckProcess;
        private int _totalCount;

        //public string ItemSourceTargetTypeName { set; get; }

        public System.Collections.IEnumerable ItemsSource
        {
            get
            {
                return TreeView.ItemsSource;
                
            }
            set
            {
                TreeView.ItemsSource = value;

                List<TreeViewModel> list = value as List<TreeViewModel>;

                _totalCount = GetCount(list);

                LabelCount.Text = string.Format("{0} / {1}", 0, _totalCount);
            }
        }

        #region ItemSourceTargetTypeName
        /// <summary>
        /// Dependency Property - ItemSourceTargetTypeName
        /// </summary>
        public static readonly DependencyProperty ItemSourceTargetTypeNameProperty =
                        DependencyProperty.Register("ItemSourceTargetTypeName", typeof(string), typeof(CheckTreeView),
                        new FrameworkPropertyMetadata("Available Categories", FrameworkPropertyMetadataOptions.BindsTwoWayByDefault));

        /// <summary>
        /// Gets or sets the header text of the check list
        /// </summary>
        public string ItemSourceTargetTypeName
        {
            get { return (string)GetValue(ItemSourceTargetTypeNameProperty); }
            set { SetValue(ItemSourceTargetTypeNameProperty, value); }
        }

        #endregion

        #region Header

        /// <summary>
        /// Dependency Property - Header
        /// </summary>
        public static readonly DependencyProperty HeaderProperty =
                        DependencyProperty.Register("Header", typeof(string), typeof(CheckTreeView),
                        new FrameworkPropertyMetadata("Available Categories", FrameworkPropertyMetadataOptions.BindsTwoWayByDefault));

        /// <summary>
        /// Gets or sets the header text of the check list
        /// </summary>
        public string Header
        {
            get { return (string)GetValue(HeaderProperty); }
            set { SetValue(HeaderProperty, value); }
        }

        #endregion
        public CheckTreeView()
        {
            InitializeComponent();
        }

        void CheckTreeView_NodeCheckedChanged(object sender, NodeEventArgs e)
        {
            UpdateCounter();
        }
        void CheckTreeView_SelectedNodesCollectionChanged(object sender, NodeSelectionEventArgs e)
        {
            if (e.CurrentSelectedNodes[e.CurrentSelectedNodes.Count - 1].IsChecked == true)
            {
                e.CurrentSelectedNodes[e.CurrentSelectedNodes.Count - 1].IsChecked = false;
               // e.CurrentSelectedNodes[e.CurrentSelectedNodes.Count - 1].Style = this.Resources["Unchecked"] as Style;//nodestyle;

            }
            else
            {
                e.CurrentSelectedNodes[e.CurrentSelectedNodes.Count - 1].IsChecked = true;
               // e.CurrentSelectedNodes[e.CurrentSelectedNodes.Count - 1].Style = this.Resources["Checked"] as Style;//nodestyle;
            }
        }
        private void TreeView_OnInitializeNode(object sender, InitializeNodeEventArgs e)
        {
//            e.Node.IsExpanded = true;
        }

        private void CheckTreeView_OnDataContextChanged(object sender, DependencyPropertyChangedEventArgs e)
        {

        }

        private void CheckTreeView_OnLoaded(object sender, RoutedEventArgs e)
        {
           

         
        }

        private int GetCount(List <TreeViewModel> treeItems)
        {
            int i = 0;
            if (treeItems != null)
            {
                foreach (var item in treeItems)
                {
                    if (item != null)
                    {
                        if (item.Children != null && item.Children.Count > 0)
                        {
                            i += GetCount(item.Children);
                        }
                        else
                            i ++;
                    }
                }
            }

            return i;
        }

        private int GetCheckedItemCount(XamDataTreeNodesCollection treeItems)
        {
            int i = 0;
            if (treeItems != null)
            {
                foreach (var item in treeItems)
                {
                    if (item != null)
                    {
                        if (item.Nodes != null && item.Nodes.Count > 0)
                        {
                            i += GetCheckedItemCount(item.Nodes);
                        }
                        else if (item.IsChecked == true)
                        {
                            i++;
                        }
                    }
                }
            }

            return i;
        }

        
        /// <summary>
        /// Update the counter label with the current number of selected items
        /// </summary>
        private void UpdateCounter()
        {
            // Update the Counter TextBlock
            if (LabelCount != null)
            {

                int checkedItemcount = GetCheckedItemCount(TreeView.Nodes);// _summary.Items.Count;
                int total = _totalCount;//TreeView.Nodes.Count;


                //List<string> names = new List<string>();
                //names.Clear();

                
                LabelCount.Text = string.Format("{0} / {1}", checkedItemcount, total);

                // Update the indeterminate state of the checkall checkbox
                if (checkedItemcount > 0 && checkedItemcount != total && _inCheckProcess == false)
                    CheckAll.IsChecked = null;
                else
                {
                    if (checkedItemcount == total)
                        CheckAll.IsChecked = true;
                    else if (checkedItemcount == 0)
                        CheckAll.IsChecked = false;
                }
            }
        }
        private void CheckAll_OnChecked(object sender, RoutedEventArgs e)
        {
            // Set a flag to indicate we're in the middle of a CheckAll process
			_inCheckProcess = true;

            foreach (XamDataTreeNode topNode in TreeView.Nodes)
            {
                topNode.IsChecked = true;
            }
            
			_inCheckProcess = false;
        }

        private void CheckAll_OnUnchecked(object sender, RoutedEventArgs e)
        {
            // Set a flag to indicate we're in the middle of a CheckAll process
            _inCheckProcess = true;

            foreach (XamDataTreeNode topNode in TreeView.Nodes)
            {
                topNode.IsChecked = false;
            }
            
            _inCheckProcess = false;
        }

        private void CheckAll_OnClick(object sender, RoutedEventArgs e)
        {
            CheckBox checkbox = (CheckBox)sender;
            if (checkbox.IsChecked == null)
                checkbox.IsChecked = false;
        }


        private void TreeView_OnLoaded(object sender, RoutedEventArgs e)
        {
            UpdateCounter();
            Loop();
        }

        public void Loop()
        {

            Action<XamDataTreeNodesCollection> accessNodes = null;

            accessNodes = (nodes) =>
            {

                foreach (var n in nodes)
                {

                    if (n.Manager.ParentNode != null)
                    {

                        n.Manager.ParentNode.ValidateParentCheckedState();

                    }

                    accessNodes(n.Nodes);

                };

            };

            accessNodes(TreeView.Nodes);

        }


    }
}
